"""
Traverse every script file in the dataset.
You can specify the method how program read, process and filter the scripts.
"""
import os
from typing import Any, List, Optional

from utils import FileManager


class TraverseDatasetItems:
    @staticmethod
    def item_reader(file_abs_path: str) -> Any:
        """
        读取数据集元素的方法。
        默认通过文本的方式读取数据集文件。
        :param file_abs_path: 文件的绝对路径
        :return: raw data from file.
        """
        with FileManager(file_abs_path, 'r') as f:
            return f.read()

    @staticmethod
    def item_processor(**kwargs) -> Optional[Any]:
        """
        根据元素的绝对路径和 item_reader 获取的原始数据，处理数据返回 processed_item_data。
        默认打印 file_abs_path 和 raw_item_data，并直接返回 raw_item_data。
        <param traverse_dir> 遍历数据集的目录的绝对路径
        <param file_abs_path> 数据集元素文件的绝对路径
        <param raw_item_data> item_reader 获取的原始数据
        <returns> processed_item_data 处理过的元素数据 or None
        """
        print(
            f"processing:\ntraverse_dir: {kwargs['traverse_dir']}, \nfile_path: {kwargs['file_abs_path']},\nraw_data:\n{kwargs['raw_item_data']}",
            end='\n\n')
        return kwargs['raw_item_data']

    @staticmethod
    def item_filter(**kwargs) -> bool:
        """
        根据元素的绝对路径、item_reader 获取的原始数据 和 已经收集元素数据的数量，判断是否保留数据。
        默认直接返回 True，保留所有数据。
        <param traverse_dir> 遍历数据集的目录的绝对路径
        <param file_abs_path> 数据集元素文件的绝对路径
        <param processed_item_data> item_processor 处理过的元素数据
        <param length_of_result> 已经收集了元素数据的数量
        <returns> True，保留数据；False，反之。
        """
        print(
            f"filtering:\ntraverse_dir: {kwargs['traverse_dir']}, \nfile_path: {kwargs['file_abs_path']},\nprocessed_data:\n{kwargs['processed_item_data']},\nlength_of_result: {kwargs['length_of_result']}",
            end='\n\n')
        return True

    @classmethod
    def traverse_dataset(cls, traverse_dir: str, item_file_type: str) -> List[tuple]:
        """
        遍历给定数据集目录下的元素，并对其进行处理，最终返回处理的结果。如果需要自定义数据集元素的读取、处理和过滤函数，请重写对应方法。
        :param traverse_dir: 遍历数据集的目录的绝对路径
        :param item_file_type: 数据集中元素的文件类型
        :returns: list of the tuple(item_absolute_path, return result of item_processor)
        """
        if not os.path.exists(traverse_dir):
            raise FileNotFoundError(f'Dataset not found at {traverse_dir}.')

        result = []
        for root, dirs, files in os.walk(traverse_dir):
            for file in files:
                file_type = os.path.splitext(file)[1]
                if file_type == '' or file_type[1:] != item_file_type:
                    continue
                file_abs_path = os.path.join(root, file)
                raw_item_data = cls.item_reader(file_abs_path)

                processed_item_data = cls.item_processor(traverse_dir=traverse_dir,
                                                         file_abs_path=file_abs_path,
                                                         raw_item_data=raw_item_data)

                if cls.item_filter(traverse_dir=traverse_dir, file_abs_path=file_abs_path,
                                   processed_item_data=processed_item_data,
                                   length_of_result=len(result)):
                    result.append((file_abs_path, processed_item_data))
        return result



from tq_datasets.dataset_op_interface import TraverseDatasetItems
from tq_utils import my_json


def test_default_method():
    print()
    t = TraverseDatasetItems()
    t.traverse_dataset(r'E:\Workspace\Pycharm\Datasets\data\Dataset-DOWL2\DOWL2\eng\al-monitor',
                       'json')


class TraverseDOWLItems(TraverseDatasetItems):
    @staticmethod
    def item_reader(file_abs_path: str) -> str:
        print('oaushoiasdoiajsodiasoidowifoqn')
        return my_json.load_json_safe(file_abs_path)


def test_override_methods():
    print()
    t = TraverseDOWLItems()
    t.traverse_dataset(r'E:\Workspace\Pycharm\Datasets\data\Dataset-DOWL2\DOWL2\eng\al-monitor',
                       'json')



from setuptools import setup, find_packages
import os.path

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = fh.read()

VERSION = '0.0.1-3'  # 0.0.1-1, '-1'表示post，修复对应版本
DESCRIPTION = 'Some simple interface for the operations of dataset.'

setup(
    name='tq_datasets',
    version=VERSION,
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    url='https://gitee.com/torchW/tq_datasets.git',
    author='TripleQuiz',
    author_email='triple_quiz@163.com',
    license='MIT',
    keywords=['python', 'util', 'datasets', 'operation', 'interface', 'abc/factory'],
    python_requires='>=3.7, <3.12',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Intended Audience :: Developers',
        'Operating System :: Microsoft :: Windows',
        'Natural Language :: Chinese (Simplified)',
    ],
    packages=find_packages(include=['tq_datasets', 'tq_datasets.*']),
    install_requires=[  # 需要安装的第三方库及版本，e.g. 'pydot==3.0.1'
        'tq_utils>=0.2.1.post5',
    ],
    include_package_data=True,
    exclude_package_data={},
    zip_safe=False,
)

"""
打包发布步骤：
1. 测试代码
2. commit & create tag
3. 打包源分发包和轮子，命令：python setup.py sdist bdist_wheel
4. PyPI发布，命令：twine upload -r tq_datasets dist/*
"""
